/***********************************
 * assignment 1 shell              *
 * CS 488 - Spring 2003            *
 * Written 8/00 by Andrew Johnson  *
 * Edited by Arun Rao 1/22/2003    *
 ***********************************/
 
#include "data.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
	
	CVertex::CVertex(float posX, float posY):next(NULL)
	{
		pos = new float[2];
		pos[X] = posX;
		pos[Y] = posY;
		magFromZeroSquared = pos[X]*pos[X] + pos[Y]*pos[Y];
	}
	
	CVertex::CVertex(const CVertex& v)
	{
		pos = new float[2];
		pos[X] = v.pos[X];
		pos[Y] = v.pos[Y];
		magFromZeroSquared = v.magFromZeroSquared;
		next = v.next;
	}
	
	CVertex::~CVertex()
	{
		delete [] pos;
	}

	
	
	void CVertex::scale(float scaleV)
	{
		pos[X] *= scaleV;
		pos[Y] *= scaleV;
		magFromZeroSquared = pos[X]*pos[X] + pos[Y]*pos[Y];
	
	
	}


////////////////////////////////////////////////////////////////////////////////////////////////////
	CVertexList::CVertexList(void):head(NULL),tail(NULL),length(0){ }
	
	CVertexList::CVertexList(const CVertexList& vList):head(NULL),tail(NULL),length(0)	//copy constructor
	{
		printf("copying vertexlist\n");
		CVertex *itr = vList.head;
		
		
		
		for(	;itr->getNext() != vList.head;itr = itr->getNext())
		{	
			printf("copying vertex\n");
			addVertex(new CVertex(*itr));
			
		}
		
		addVertex(new CVertex(*itr));
		
	
	
	}
	
	
	
	CVertexList::~CVertexList()
	{
		CVertex *prev, *next;
		next = head;
		prev = next;
		for(;prev != tail; prev = next, next = next->getNext())
		{
			delete prev;
		}
		
		delete prev;
	
	}
	
	//add another vertex to the end of the list
	void CVertexList::addVertex(CVertex *v)
	{
		if(!v)	//make sure it's a valid vertex
			return;
			
		length++;
		printf("adding vertex at %f, %f\n",v->getX(),v->getY());
		if(!head)
		{
			head = v;
			tail = head;
			v->setNext(v);
			printf("added to head\n");
			return;
		}
		else
		{
			tail->setNext(v);
			tail = v;
			v->setNext(head);
			printf("added to end\n");
			return;
		
		}
		
	
	}
	
	//get a vertex at a specific index in the list
	CVertex * CVertexList::getVertex(int i)
	{
		if(!head)
			return NULL;
			
		CVertex *itr = head;
		
		for(int j = 0; j < i; j++)
		{
			if(!itr->getNext())
				return NULL;
			else
				itr = itr->getNext();
		
		} 
		
		return itr;
	
	}
	
	//get a vertex after a given vertex
	void CVertexList::scaleList(float scaleV)
	{
		
		if(!head)
			return;
			
		CVertex *itr = head;
		
		for(int i = 0; itr && i < length;i++, itr = itr->getNext())
		{
			printf("scaling vertex at: %d\n",itr);
			itr->scale(scaleV);
		
		} 
		
	
	
	}
	
	

////////////////////////////////////////////////////////////////////////////////////////////////////

	void CSpaceObject::copyVertexList(CVertexList *vL)
	{
		if(!vL)
			return;
			
		if(vertexList)
			delete vertexList;

		vertexList = new CVertexList(*vL);
		
		avgRadius = 0;

		CVertex *temp = vertexList->getVertex(0);
		int i = 0;
			for(; i < vertexList->getLength(); i++, temp = vertexList->getNextVertex(temp))
			{
				avgRadius += temp->getMagFromZero();
			}
		avgRadius /= i;
	}
	
	
	CSpaceObject::CSpaceObject()
	{
		vertexList = NULL;
		
		xVelocity = 0;
		yVelocity = 0;
		
		position = new CVertex(0,0);
		
		direction = new CVertex(0,1);
		
		avgRadius = 0;
		canTakeDamage = true;
		red = green = blue = 1;

		assocWorld = NULL;
	
	}
	
	void CSpaceObject::setSpeed(float sp)
	{
		if(!xVelocity)
		{	if(yVelocity < 0)
				yVelocity = -sp;
			else if(yVelocity > 0)
				yVelocity = sp;
		}
		else
		{
			float rad = atanf(yVelocity/xVelocity);
			xVelocity = sp*cosf(rad);
			yVelocity = sp*sinf(rad);

		}
	}

////////////////////////////////////////////////////////////////////////////////////////////////////
CAsteroid::CAsteroid():next(NULL),collidedWithMissle(false),theta(rand()),damageLevel(LEVEL0)
	{
		CSpaceObject();
		rotateDirection = (rand() % 2)? ASTEROID_ROT_CCW : ASTEROID_ROT_CW;
		red = AsteroidColor[RED];
		green = AsteroidColor[GREEN];
		blue = AsteroidColor[BLUE];
	
		srand(time(NULL));

		xVelocity = rand();
		yVelocity = rand();

		xVelocity = xVelocity * pow(-1,(int)rand() % 2);
		yVelocity = yVelocity * pow(-1,(int)rand() % 2);
		
		setSpeed(ASTEROID_SPEED);
		
		

	}
	
	CAsteroid::CAsteroid(CVertexList *vList):next(NULL),collidedWithMissle(false),theta(0),damageLevel(LEVEL0)
	{
		CSpaceObject();
		
		copyVertexList(vList);
		
		rotateDirection = (rand() % 2)? ASTEROID_ROT_CCW : ASTEROID_ROT_CW;
		
		
		
		red = AsteroidColor[RED];
		green = AsteroidColor[GREEN];
		blue = AsteroidColor[BLUE];
		
		xVelocity = rand();
		yVelocity = rand();
		
		setSpeed(UNITS_PER_SECOND(40));
		
		xVelocity = xVelocity * pow(-1,(int)rand() % 2);
		yVelocity = yVelocity * pow(-1,(int)rand() % 2);
	}
	
	void CAsteroid::move()
	{
		float xpos, ypos;
		xpos = position->getX() + xVelocity;
		ypos = position->getY() + yVelocity;
		
		if(xpos + avgRadius >= RIGHT_EDGE)
		{
			xpos = RIGHT_EDGE - avgRadius;
			xVelocity *= -1;
		
		}
		else if(xpos - avgRadius <= LEFT_EDGE)
		{
			xpos = LEFT_EDGE + avgRadius;
			xVelocity *= -1;
		}
	
		if(ypos + avgRadius >= BOTTOM_EDGE)
		{
			ypos = BOTTOM_EDGE - avgRadius;
			yVelocity *= -1;
		}
		else if(ypos - avgRadius <= TOP_EDGE)
		{
			ypos = TOP_EDGE + avgRadius;
			yVelocity *= -1;
		}
		
		if(rotateDirection == ASTEROID_ROT_CCW)
			theta += ASTEROID_THETA;
		else
			theta -= ASTEROID_THETA;
		
		
		position->setPosition(xpos,ypos);	
		
	}
	
	
	void CAsteroid::takeDamage( void )
	{ 
		collidedWithMissle = true; 
		switch(damageLevel)
		{
		case LEVEL0:
			damageLevel = LEVEL1;
			break;
		case LEVEL1:
		default:
			damageLevel = DESTROYED;
			break;
		}
	}
	
	
	 
	CAsteroid* CAsteroid::spawnChildren(void)
	{
		if(damageLevel == DESTROYED)
			return NULL;

		CAsteroid *child1, *child2;
		
		printf("scaling vertexList\n");
		vertexList->scaleList(.5);
		printf("creating children\n");
		child1 = new CAsteroid(vertexList);
		child2 = new CAsteroid(vertexList);
		printf("child1 address %d, child2 address %d\n",child1,child2);
		child1->next = child2;
		
		child1->setColor(red,green,blue);
		child2->setColor(red,green,blue);

		child1->setRotation(ASTEROID_ROT_CCW);
		child2->setRotation(ASTEROID_ROT_CW);

		/*child1->xVelocity = 1;
		child2->xVelocity = -1;
		child1->yVelocity = 1;
		child2->yVelocity = -1;*/
		
		
		
		child1->setDamageLevel(damageLevel);
		child2->setDamageLevel(damageLevel);
		
		child1->position->setPosition(position->getX() + 15, position->getY() + 15);
		child1->move();
		child2->position->setPosition(position->getX() - 15, position->getY() - 15);
		child2->move();
		
		return child1;
		
	
	}
	
	void CAsteroid::draw()
	{
		
		glPushMatrix();
		//glLoadIdentity();
			glColor3f(red,green,blue);
			glTranslatef(position->getX(),position->getY(),0);
			glRotatef(theta,0,0,1);
			//glTranslatef(-(position->getX()), -(position->getY()),0);
			
			glBegin(GL_LINE_LOOP);
				CVertex* temp = vertexList->getVertex(0);
				for(int i = 0; i < vertexList->getLength(); i++, temp = vertexList->getNextVertex(temp))
					glVertex2f(temp->getX(),temp->getY());
			//glLoadIdentity();		
			glEnd();
			
		glPopMatrix();	
		//
		
	}




////////////////////////////////////////////////////////////////////////////////////////////////////
CWorld::CWorld():asteroidList(NULL)
	{
	
		
	
	}
	
	bool CWorld::addAsteroid(CAsteroid* ast)
	{
		//put asteroid at front of list
		ast->setNext(asteroidList);		
		asteroidList = ast;
		asteroidList->setAssocWorld(this);
		asteroidList->setColor(AsteroidColor[RED],AsteroidColor[GREEN],AsteroidColor[BLUE]);
		return (asteroidList == ast);
	}
	
	void CWorld::updateAsteroidList( void )
	{
		if(!asteroidList)
			return;
			
		CAsteroid *temp = asteroidList;
		CAsteroid *prev = temp;
		printf("updating asteroid list\n");
		for(	; temp; )
		{
			printf("temp at %d\n",temp);
			if(temp->isDamaged())
			{	
				printf("asteroid at %d is damaged\n",temp);
				//have temp spawn two children
				CAsteroid* children = temp->spawnChildren();
				
				//place the next pointer of the previous to the asteroid after temp
				prev->setNext(temp->getNext());
				
				if(children)
				{
					//place the children in the middle of the list
					children->getNext()->setNext(temp->getNext());
					
					//set temp to prev so that we can iterate through the list with no problem
					printf("temp: %d, child2->next: %d, temp->next: %d\n",temp,children->getNext()->getNext(),
								temp->getNext());
				
					//if we are at the beginning then fix the head pointer otherwise fix next pointer
					if(temp == asteroidList)
					{
						asteroidList = children;
					}
					else
						prev->setNext(children);

					delete temp;

					//have temp go the 2nd child
					temp = children->getNext();
				}
				else
				{
					if(temp == asteroidList)
						prev = asteroidList = asteroidList->getNext();
					else
						prev->setNext(temp->getNext());

					delete temp;

					temp = prev;
					
					
					printf("asteroidList: %d, temp %d\n", asteroidList, temp);



				}
				
				
				NumAsteroidsDestroyed++;
				AsteroidDestroyed = true;
				
				
				
				
			}

			//step forward if possible
			if(temp)
			{
				prev = temp;
				temp = temp->getNext();
			}

		}

		/*for(temp = asteroidList	; temp; prev = temp, temp = temp->getNext())
		{
			printf("temp at %d\n",temp);
		}*/
		
		//if there are  no more asteroids then create the set amount of asteroids on the screen.
		if(!asteroidList)
		{
			//Give the rocket temporary invincibility for passing the level
			assocRocket->temporaryInvincible();
			for(unsigned int i = 0; i < NUM_ASTEROIDS; i++)
    			{
				CAsteroid* temp = new CAsteroid(DefaultAsteroidVerts);
				printf("new asteroid at %d\n",temp);
				temp->setPosition((abs(rand()) % RIGHT_EDGE) * (abs(rand()) % 2 + -1),
					(abs(rand()) % BOTTOM_EDGE) * (abs(rand()) % 2 + -1));
				addAsteroid(temp);
		
			}		
		
		
		
		
		}
	
	
	}
	
	void CWorld::drawAsteroids(void)
	{
		if(!asteroidList)
			return;
			
		CAsteroid *temp = asteroidList;
		
		for( ; temp; temp = temp->getNext())
		{
			temp->draw();
		
		}
		
		
	}

	void CWorld::moveAsteroids(void)
	{
		if(!asteroidList)
			return;
			
		CAsteroid *temp = asteroidList;
		
		for( ; temp; temp = temp->getNext())
		{
			temp->move();
		
		}


	}
	
	
	


////////////////////////////////////////////////////////////////////////////////////////////////////
CMissle::CMissle( ):inMissleBay(true)
	{
		CSpaceObject();
		red = green = (blue = 0) + 1;	//default color yellow
		speed = UNITS_PER_SECOND(100);
		setExplosionColor();
	
	}
	
	
		 
	bool CMissle::fire( float x, float y, float xDir, float yDir )
	{
		//can't fire if not in a world
		if(!assocWorld)
			return false;
		else if(inMissleBay)	//only can fire if in missle bay, if it's already fired, can't fire it again
		{
			pos[X] = x;
			pos[Y] = y;
			xVelocity = xDir * speed;
			yVelocity = yDir * speed;
			missleAnimState = NO_EXPLODE;
			printf("missle go!");
			return !(inMissleBay = false);
		}
		
		return false; //incase nothing works out	
				
	}
	
	
	void CMissle::draw(void)
	{
		//if it's still in the missle bay or there is not associated world then forget it
		if(inMissleBay || !assocWorld)
			return;
		
		
		if(missleAnimState == NO_EXPLODE)
		{
			glBegin(GL_LINE_LOOP);
				glColor3f(red,green,blue);
				glVertex2f(pos[X]-1,pos[Y]-1);
				glVertex2f(pos[X]+1,pos[Y]-1);
				glVertex2f(pos[X]+1,pos[Y]+1);
				glVertex2f(pos[X]-1,pos[Y]+1);	
		
			glEnd();
		}
		else if(missleAnimState == EXPLODE_ANIM)
		{
			if( time(NULL) >= animEndTime)
			{
				//explosion is over
				missleAnimState = EXPLODE_DONE;
				glBegin(GL_LINE_LOOP);
					glColor3f(red,green,blue);
					glVertex2f(pos[X]-1,pos[Y]-1);
					glVertex2f(pos[X]+1,pos[Y]-1);
					glVertex2f(pos[X]+1,pos[Y]+1);
					glVertex2f(pos[X]-1,pos[Y]+1);	
		
				glEnd();
			}
			else
			{
				//draw the explosion
				glBegin(GL_LINE_LOOP);
					//color at the opposite end of spectrum compared to color of asteroid.
					glColor3f(expRed,expGreen,expBlue);
		
					glVertex2f(pos[X],pos[Y]+10);
					glVertex2f(pos[X]+5,pos[Y]+7.5);
					glVertex2f(pos[X]+10,pos[Y]+10);
					glVertex2f(pos[X]+7.5,pos[Y]+.5);
					glVertex2f(pos[X]+10,pos[Y]);
					////////////////////////////////
					glVertex2f(pos[X]+7.5,pos[Y]-5);
					glVertex2f(pos[X]+10,pos[Y]-10);
					glVertex2f(pos[X]+5,pos[Y]-7.5);
					glVertex2f(pos[X],pos[Y]-10);
					
					
					////////////////////////////////
					glVertex2f(pos[X]-5,pos[Y]-7.5);
					glVertex2f(pos[X]-10,pos[Y]-10);
					glVertex2f(pos[X]-7.5,pos[Y]-5);
					glVertex2f(pos[X]-10,pos[Y]);
					////////////////////////////////
					glVertex2f(pos[X]-7.5,pos[Y]+5);
					glVertex2f(pos[X]-10,pos[Y]+10);
					glVertex2f(pos[X]-5,pos[Y]+7.5);
					
				glEnd();
				
				
				printText(boomTextPos[X],boomTextPos[Y]-10,.05,BOOM_TEXT,textDir);
					
					
					

			}



		}
	
	}
	
	//move the missle
	void CMissle::move(void)
	{
		//if it's still in the missle bay or there is not associated world to do collision detection with then forget it
		if(inMissleBay || !assocWorld)
			return;
		//if the explosion is being animated then don't move
		if(missleAnimState == EXPLODE_ANIM)
		{
					
			return;
		}
		else if(missleAnimState == EXPLODE_DONE)//missle done exploding, move to missle bay
		{
			missleAnimState = NO_EXPLODE;
			red = MissleColor[RED];
			green = MissleColor[GREEN];
			blue = MissleColor[BLUE];
			inMissleBay = true;
			return;

		}


		pos[X] += xVelocity;
		pos[Y] += yVelocity;
		
		//outside framebuffer?
		//if so then set positions of missle and text, set the missle state and determine the time that the explosion
		//will last
		if(pos[X] <= LEFT_EDGE) 
		{
			pos[X] = LEFT_EDGE;
			missleAnimState = EXPLODE_ANIM;
			animEndTime = time(NULL) + 1;

			//place the text position
			boomTextPos[X] = LEFT_EDGE;

			if((boomTextPos[Y] =(pos[Y] - 5*strlen(BOOM_TEXT)))+ 10*strlen(BOOM_TEXT) >= BOTTOM_EDGE)
				boomTextPos[Y] = BOTTOM_EDGE - 10*strlen(BOOM_TEXT);
			else if(boomTextPos[Y] <= TOP_EDGE)
				boomTextPos[Y] = TOP_EDGE;
			
			textDir = DOWN;
			
			return;
		}
		else if(pos[X] >= RIGHT_EDGE)
		{
			pos[X] = RIGHT_EDGE;
			missleAnimState = EXPLODE_ANIM;
			animEndTime = time(NULL) + 1; 
			//place the text position
			boomTextPos[X] = pos[X] - glutStrokeWidth(GLUT_STROKE_MONO_ROMAN,BOOM_TEXT[0])*.05;
			if((boomTextPos[Y] =(pos[Y] + 5*strlen(BOOM_TEXT))) + 10*strlen(BOOM_TEXT) >= BOTTOM_EDGE)
				boomTextPos[Y] = BOTTOM_EDGE - 10*strlen(BOOM_TEXT);
			else if(boomTextPos[Y] <= TOP_EDGE)
				boomTextPos[Y] = TOP_EDGE;
			
			textDir = DOWN;
			

			return;
		}
		else if(pos[Y] <= TOP_EDGE)
		{
			missleAnimState = EXPLODE_ANIM;
			animEndTime = time(NULL) + 1;
			pos[Y] = TOP_EDGE;

			boomTextPos[Y] = TOP_EDGE + 30;

			if((boomTextPos[X] = (pos[X] - (.05*glutStrokeWidth(GLUT_STROKE_MONO_ROMAN,BOOM_TEXT[0])*strlen(BOOM_TEXT)/2)))
					<= LEFT_EDGE)
						boomTextPos[X] = LEFT_EDGE;
			textDir = RIGHT;
			
	
			return;
		}
		else if(pos[Y] >= BOTTOM_EDGE)
		{
			missleAnimState = EXPLODE_ANIM;
			animEndTime = time(NULL) + 1;
			pos[Y] = BOTTOM_EDGE;
			
			boomTextPos[Y] = BOTTOM_EDGE - 10;

			if((boomTextPos[X] = (pos[X] - (.05*glutStrokeWidth(GLUT_STROKE_MONO_ROMAN,BOOM_TEXT[0])*strlen(BOOM_TEXT)/2)))
					<= LEFT_EDGE)
						boomTextPos[X] = LEFT_EDGE;
			textDir = RIGHT;
			
			
			
			return;
		

		}
		
		
		
		//collision detection
		CAsteroid *tempAst = assocWorld->getAsteroids();	
		CVertex* tempVertx = NULL;
		if(!tempAst)
			return;
		else
		{
			float deltaX, deltaY;
			for(	;tempAst; tempAst = tempAst->getNext())
			{
				tempVertx = tempAst->getCenter();

				deltaX = tempVertx->getX() - pos[X];
				deltaY = tempVertx->getY() - pos[Y];
				
				if((deltaX*deltaX + deltaY*deltaY) < (tempAst->getAvgRadius()*tempAst->getAvgRadius()))
				{
					//collision
					printf("collision with asteroid\n");
					tempAst->takeDamage();
					assocWorld->updateAsteroidList();
					missleAnimState = EXPLODE_ANIM;
					animEndTime = time(NULL) + 1;
					boomTextPos[X] = pos[X];
					boomTextPos[Y] = pos[Y];
					textDir = RIGHT;
					NumAsteroidDestroyedByMissle++;
					break;
				}
	
			}//end for loop
		
		
		}//end else
	
		//no collisions!
	
	}

	void CMissle::setExplosionColor(void)
	{
			expRed = 1 - AsteroidColor[RED];
			if(expRed == .5)
				expRed = 1;
			else if(!expRed)
				expRed = .5;

			expGreen = 1 - AsteroidColor[GREEN];
			if(expGreen == .5)
				expGreen = 1;
			else if(!expGreen)
				expGreen = .5;

			expBlue = 1 - AsteroidColor[BLUE];
			if(expBlue == .5)
				expBlue = 1;
			else if(!expBlue)
				expBlue = .5;

	}


////////////////////////////////////////////////////////////////////////////////////////////////////
CRocket::CRocket(CVertexList*& vL):invincible(true),endInvincibleTime(time(NULL) + 6)
	{
		CSpaceObject();
		
		for(int i = 0; i < MISSLE_AMMO; i++)
		{
			missleBay[i] = new CMissle();
			missleBay[i]->setColor(MissleColor[RED],MissleColor[GREEN],MissleColor[BLUE]);
			red = RocketColor[RED];
			green = RocketColor[GREEN];
			blue = RocketColor[BLUE];
		}
		
		copyVertexList(vL);
		setMaxSpeed(80);		
	}
	
	
	//set the associated world and set the assocworld of the missles
	void CRocket::setAssocWorld(CWorld *wPtr)
	{
		CSpaceObject::setAssocWorld(wPtr);
		for(int i = 0; i < MISSLE_AMMO; i++)
			missleBay[i]->setAssocWorld(wPtr);
	
	}
	
	void CRocket::draw(void)
	{
		if(!vertexList)
			return;
			
		if(!invincible)
		{
			glBegin(GL_LINE_LOOP);
				glColor3f(red,green,blue);
		
				CVertex *temp = vertexList->getVertex(0);
				for(int i = 0; i < vertexList->getLength(); temp = vertexList->getNextVertex(temp), i++)
				{
					glVertex2f(temp->getX(),temp->getY());
				}
			
		
			glEnd();
		}
		else
		{
			//draw doppelgangers of the ship
			
			
			
			glBegin(GL_LINE_LOOP);
				glColor3f(1- red,green,blue);
			
				CVertex *temp = vertexList->getVertex(0);
				for(int i = 0; i < vertexList->getLength(); temp = vertexList->getNextVertex(temp), i++)
				{
					glVertex2f(temp->getX(),temp->getY());
				}
			
		
			glEnd();
			
			glBegin(GL_LINE_LOOP);
				glColor3f(red,1- green,blue);
			
				temp = vertexList->getVertex(0);
				for(int i = 0; i < vertexList->getLength(); temp = vertexList->getNextVertex(temp), i++)
				{
					glVertex2f(temp->getX() - direction->getX()*2,temp->getY() - direction->getY()*2);
				}
			
		
			glEnd();
			
			glBegin(GL_LINE_LOOP);
				glColor3f(red,green,1- blue);
			
				temp = vertexList->getVertex(0);
				for(int i = 0; i < vertexList->getLength(); temp = vertexList->getNextVertex(temp), i++)
				{
					glVertex2f(temp->getX() + direction->getX()*2,temp->getY() + direction->getY()*2);
				}
			
		
			glEnd();
			
			
			if(time(NULL) >= endInvincibleTime)
				invincible = false;
		
		
		}
		
		
		//draw missles
		for(int i = 0; i < MISSLE_AMMO; i++)
			missleBay[i]->draw();
		
		
			
	
	
	}
	
	
	
	void CRocket::move()
	{
		
		loadIdentity();
		pushMatrix();
		
			float tempPos[2];
			tempPos[X] = position->getX();
			tempPos[Y] = position->getY();

			
			//determine if we are outside the bounds of the game screen
			if(position->getX() - avgRadius < LEFT_EDGE)
			{
				position->setPosition(LEFT_EDGE + avgRadius,position->getY());
				translateMatrix(position->getX() - tempPos[X],0);
				position->setPosition(tempPos[X],position->getY());
				xVelocity *= -1;
			}
			else if(position->getX() + avgRadius > RIGHT_EDGE)
			{
				position->setPosition(RIGHT_EDGE - avgRadius, position->getY());
				translateMatrix(position->getPosition()[X] - tempPos[X],0);
				position->setPosition(tempPos[X],position->getY());
				xVelocity *= -1;
			}
			else
			{
				translateMatrix(xVelocity,0);
			}

			if(position->getY() - avgRadius < TOP_EDGE)
			{	
				position->setPosition(position->getX(), TOP_EDGE + avgRadius);
				translateMatrix(0,position->getY() - tempPos[Y]);
				position->setPosition(position->getX(), tempPos[Y]);
				yVelocity *= -1;
				
			}
			else if(position->getY() + avgRadius > BOTTOM_EDGE)
			{
				position->setPosition(position->getX(), BOTTOM_EDGE - avgRadius);
				translateMatrix(0,position->getY() - tempPos[Y]);
				position->setPosition(position->getX(), tempPos[Y]);
				yVelocity *= -1;
				
			}
			else
			{
				translateMatrix(0,yVelocity);
			}
			
			//collision detection
			CAsteroid *tempAst = assocWorld->getAsteroids();	
			CVertex* tempVertx = NULL;
			
			if(tempAst && !invincible)
			{
				float deltaX, deltaY;
				for(	;tempAst; tempAst = tempAst->getNext())
				{
					tempVertx = tempAst->getCenter();
	
					deltaX = tempVertx->getX() - position->getX();
					deltaY = tempVertx->getY() - position->getY();
				
					if((deltaX*deltaX + deltaY*deltaY) < (tempAst->getAvgRadius()*tempAst->getAvgRadius()))
					{
						//collision
						printf("collision with asteroid\n");
						tempAst->takeDamage();
						assocWorld->updateAsteroidList();
						invincible = true;
						endInvincibleTime = time(NULL) + 6;//give six seconds of invincibility
						
						//put the ship back at the center
						//hack!!!!  Not good code writing, but necessary!!!!
						popMatrix();
						pushMatrix();
						translateMatrix(-(position->getX()),-(position->getY()));
						
						xVelocity = 0;
						yVelocity = 0;
						
						NumShipsDestroyed++;
						ShipDestroyed = true;

						break;
					}
	
				}//end for loop
		
			
			}//end else
	
			//no collisions!
			
			CVertex *temp = vertexList->getVertex(0);
			for(  int i = 0; i < vertexList->getLength(); i++, temp = vertexList->getNextVertex(temp))
			{
				if(temp)
				{
					
					temp->setPosition(transformPoint(*(TransformMatrixStack.top),temp->getPosition()));
					//printf("move vertex %d to: (%f, %f)\n",i,temp->getX(), temp->getY());
				}
			
			}

			position->setPosition(transformPoint(*(TransformMatrixStack.top), position->getPosition()));

		popMatrix();
		
	
	}
	
	//accelerate in the direction the ship is facing
	void CRocket::thrustInDirection()
	{
		xVelocity += direction->getX();
		yVelocity += direction->getY();
		
		if(xVelocity*xVelocity + yVelocity*yVelocity > ROCKET_MAX_SPEED*ROCKET_MAX_SPEED)
		{
			xVelocity = direction->getX() * ROCKET_MAX_SPEED;
			yVelocity = direction->getY() * ROCKET_MAX_SPEED;
			printf("%f in x - %f in y\n",xVelocity,yVelocity);
		
		}
		//
	
	}

	//rotate the rocket about its own origin
	void CRocket::rotateRocket(float theta)
	{
		//printf("rotating");
		loadIdentity();
		pushMatrix();
			translateMatrix(position->getX(),position->getY());
			rotateMatrix(theta);
			translateMatrix(-position->getX(),-position->getY());
			CVertex *temp = vertexList->getVertex(0);
			for(  int i = 0; i < vertexList->getLength(); i++, temp = vertexList->getNextVertex(temp))
			{
				if(temp)
				{
					
					temp->setPosition(transformPoint(*(TransformMatrixStack.top),temp->getPosition()));
					//printf("move vertex %d to: (%f, %f)\n",i,temp->getX(), temp->getY());
				}
			
			}

		popMatrix();
		pushMatrix();
			rotateMatrix(theta);
				direction->setPosition(transformPoint(*(TransformMatrixStack.top),direction->getPosition()));
				//printf("direction %f, %f\n", direction->getPosition()[X], direction->getPosition()[Y]);
		popMatrix();

	}
	
	//shoot an available missle
	void CRocket::shootMissle(void)
	{
		printf("shoot!\n");
		for(int i = 0; i < MISSLE_AMMO; i++)
		{	//if the missle is in the bay and it fires then break, and increment the number of shots taken
			if(missleBay[i]->inBay())
				if(missleBay[i]->fire(position->getX() + direction->getX()*avgRadius,
										position->getY() + direction->getY()*avgRadius,
										direction->getX(),direction->getY()))
				{
					NumberShots++;
					break;
				}
		
		
		}
	
	
	}
	
	void CRocket::updateMissles(void)
	{
		for(int i = 0; i < MISSLE_AMMO; i++)
			missleBay[i]->move();
	
	
	}

	void CRocket::setMissleSpeeds(float sp)
	{
		for(int i = 0; i < MISSLE_AMMO; i++)
			missleBay[i]->setSpeed(sp);
	}

	void CRocket::temporaryInvincible(void)
	{
			invincible = true;
			endInvincibleTime = time(NULL) + 6;//give six seconds of invincibility
						

	}




/************************************************************
* printText
*
* prints out text to screen
*************************************************************/
void printText(float x, float y, float scale, char* text,int dir)
{
	if(!text)
		return;
		
	int length = strlen(text);
	
	glPushMatrix();
		glTranslatef(x,y,0);
		glScalef(scale,-scale,1);
		if(dir == RIGHT)
		{
			for(int i = 0; i < length; i++)
			{
				glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN,text[i]);
				//glTranslatef(glutStrokeWidth(GLUT_STROKE_MONO_ROMAN,text[i]),0,0);

			}
		}
		else if(dir == DOWN)
		{
			for(int i = 0; i < length; i++)
			{
				glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN,text[i]);
				glTranslatef(-glutStrokeWidth(GLUT_STROKE_MONO_ROMAN,text[i]),-125,0);

			}



		}

	glPopMatrix();
}


/************************************************************
* intToChar
*
* returns a character pointer to a string made from given int
*************************************************************/
char* intToChar(int num)
{
	static length;
	
	length = 0;
	
	if(num == 0)
		return "0";

	for(int temp = num; temp; temp /= 10)
		length++;
		
	char *text = new char[length + 1];
	
	text[length] = '\0';
	
	printf("text: %s\n", text);
	
	for(int i = length - 1; i >= 0; i--, num /= 10)
		text[i] = num % 10 + 48;
	
	printf("text: %s\n", text);
		
	return text;



}

/************************************************************
* getAngleRad
*
* returns the angle in radians from org to pos
*************************************************************/
float getAngleRad(float *& pos, float *& org)
{
	float x = pos[X] - org[X];
	float y = pos[Y] - org[Y];
	
	return atanf(x / y);


}

/**********************************************************
* loadIdenity
* place the identity matrix into the current matrix
***********************************************************/
void loadIdentity(void)
{
	STransform *temp = TransformMatrixStack.top;
	
	if(temp)
	{
		
		temp->matrix[0][0] = 1;
		temp->matrix[0][1] = 0;
		temp->matrix[0][2] = 0;
		temp->matrix[1][0] = 0;
		temp->matrix[1][1] = 1;
		temp->matrix[1][2] = 0;
		temp->matrix[2][0] = 0;
		temp->matrix[2][1] = 0;
		temp->matrix[2][2] = 1;
	
		
	
	}

}

/**********************************************************
* getIdenityMatrix
* return a copy of the identity matrix 
***********************************************************/
STransform * getIdentityMatrix(void)
{
	STransform *temp = new STransform();
	
	
		
		temp->matrix[0][0] = 1;
		temp->matrix[0][1] = 0;
		temp->matrix[0][2] = 0;
		temp->matrix[1][0] = 0;
		temp->matrix[1][1] = 1;
		temp->matrix[1][2] = 0;
		temp->matrix[2][0] = 0;
		temp->matrix[2][1] = 0;
		temp->matrix[2][2] = 1;
		
		
		temp->next = NULL;
	
	
	return temp;

}

/********************************************************
* scaleMatrix
* scale the matrix on top of TransformMatrixStack by the x, y values given
*******************************************************/
void scaleMatrix(float x,float y)
{
	STransform *temp = TransformMatrixStack.top;
	
		temp->matrix[0][0] *= x;
		temp->matrix[1][0] *= y;
}

/************************************************************
* translateMatrix
* translate the matrix on top of the stack by the given x y values
****************************************************************/
void translateMatrix(float x, float y)
{
	STransform *temp = TransformMatrixStack.top;
	
		
		temp->matrix[0][2] = temp->matrix[0][0] * x + temp->matrix[0][1] * y + temp->matrix[0][2] * 1;
		temp->matrix[1][2] = temp->matrix[1][0] * x + temp->matrix[1][1] * y + temp->matrix[1][2] * 1;
			
}

/*****************************************************************
* rotateMatrix
* set the rotation of the matrix on the top of the stack
*******************************************************************/
void rotateMatrix(float theta)
{
	STransform *temp = TransformMatrixStack.top;

		float pos00,pos01,pos10,pos11;
		pos00 = (temp->matrix[0][0] * cosf(theta)) + (temp->matrix[0][1] * sinf(theta));
		pos01 = (temp->matrix[0][0] * -sinf(theta)) + (temp->matrix[0][1] * cosf(theta));
		pos10 = (temp->matrix[1][0] * cosf(theta)) + (temp->matrix[1][1] * sinf(theta));
		pos11 = (temp->matrix[1][0] * -sinf(theta)) + (temp->matrix[1][1] * cosf(theta));

		temp->matrix[0][0] =  pos00;
		temp->matrix[0][1] =  pos01;
		temp->matrix[1][0] =  pos10;
		temp->matrix[1][1] =  pos11;
}


/*****************************************************************
* pushMatrix
* places a copy of the current matrix onto the top of the stack
*****************************************************************/
void pushMatrix(void)
{
	STransform *temp = new STransform();
	
		for(int i = 0; i < 3; i++)
		 	for(int j = 0; j < 3; j++)
		 		temp->matrix[i][j] = TransformMatrixStack.top->matrix[i][j];
	
		
		temp->next = TransformMatrixStack.top;
		
		TransformMatrixStack.top = temp;
	return;	

}

/*******************************************************************
* popMatrix
* pop the current matrix on top of the stack off
*******************************************************************/
void popMatrix(void)
{
	STransform *temp = TransformMatrixStack.top;
	TransformMatrixStack.top = TransformMatrixStack.top->next;
	delete temp;

}


/****************************************************************
* transformPoint
* use the transform matrix given by a, and transform point given by b
*******************************************************************/
float* transformPoint(STransform& a, const float*& b)
{
	float *pos = new float[2];
	
	pos[X] = b[X]*a.matrix[0][0] + b[Y]*a.matrix[0][1] + a.matrix[0][2];
	pos[Y] = b[X]*a.matrix[1][0] + b[Y]*a.matrix[1][1] + a.matrix[1][2];
	
	return pos;

}


/*************************************************************
 *
 * keyboard
 * 
 * deal with the keyboard input to the program.  
 * Keyboard input will move the paddle
 *************************************************************/

void keyboard(unsigned char c, int x, int y)
{
	
	
	currentTime = glutGet(GLUT_ELAPSED_TIME);

	
	if(GameState != END_SCREEN)
	{
		switch(toupper(c))
		{
		case 'Q':
			//show end screen before quitting!
			GameState = END_SCREEN;
			//process results
			if(NumberShots)
				ShotPercentage = ((float)((float)NumAsteroidDestroyedByMissle /((float)NumberShots)) * 100);
			else
				ShotPercentage = 0;
			
			if(NumShipsDestroyed >= NumAsteroidsDestroyed)
				TotalScore = 0;
			else	
				TotalScore = (float)NumAsteroidDestroyedByMissle *( (float)ShotPercentage /(float)100) -  NumShipsDestroyed;
			
			break;
		case 'K':
			Rocket->thrustInDirection();
			break;
		case 'J':
			//printf("rotate ccw\n");
			Rocket->rotateRocket(-(TURN_RATE ));// / (1000 / (currentTime - prevTime)));// * (float)(currentTime - prevTime) / ((float)1000)));
			//printf("radians: %f  keyboard delay: %d\n",(TURN_RATE * (float)(currentTime - prevTime) / ((float)1000)),currentTime - prevTime);
			break;
		case 'L':
			//printf("rotate cw\n");
			Rocket->rotateRocket((TURN_RATE));// / (1000 / (currentTime - prevTime)) ));//* (float)(currentTime - prevTime) / ((float)1000)));
			//printf("radians: %f\n",-(TURN_RATE * (float)(currentTime - prevTime) / ((float)1000)));
			
			break;
		case 'I':
			Rocket->shootMissle();
			break;
		default:
			break;
		}
	}
	else if(toupper(c) == 'Q')
		exit(0);

	prevTime = glutGet(GLUT_ELAPSED_TIME);
}


 
/*************************************************************
 *
 * update
 * 
 * update everything that needs to be updated in the scene - this
 * function should be called via a timer to ensure that updates 
 * happen at a regular rate, independent of the graphics update
 * rate
 *
 *************************************************************/

void update(int value)
{
   if(GameState != PLAYING)
		return;
    /* at the end of this function
       reinstate the timer callback */
    /********************************/

	// set the location of the triangle to bounce back
	// and forth across the window
	//printf("update called\n");
	Rocket->move();
	Root->moveAsteroids();
	
	//prevTime = glutGet(GLUT_ELAPSED_TIME);

	glutTimerFunc(UPDATE_TIME, update, 1);
	
} 

void missleUpdate(int value)
{
	if(GameState != PLAYING)
		return;

	Rocket->updateMissles();
	
	glutTimerFunc(MISSLE_UPDATE_TIME, missleUpdate,1);
	//prevTime = glutGet(GLUT_ELAPSED_TIME);

}
   
/*************************************************************
 *
 * display
 * 
 * this function is called each time the screen is to be drawn
 *************************************************************/

void display(void)
{
	
	//end screen info only!
	if(GameState == END_SCREEN)
	{
		
		glColor3f(MissleColor[RED],MissleColor[GREEN],MissleColor[BLUE]);
		printText(-100, 0, .08, "Shot Percentage...", RIGHT);
		printText(50,0, .08, intToChar(ShotPercentage), RIGHT);
		printText(75,0,.2,"%",RIGHT);
		glColor3f(RocketColor[RED],RocketColor[GREEN],RocketColor[BLUE]);
		printText(-70,45,.08, "Total Score",RIGHT);
		printText(-180,55,.06,"(Shot% * Asteroids Destroyed - Ships Lost)",RIGHT);
		printText(-200,65,.1,"--------------------------------------",RIGHT);
		glColor3f(1,1,1);
		printText(130,60,.2,intToChar(TotalScore),RIGHT);
		glutSwapBuffers();
		return;
	}


	//Otherwise we don't draw anything unless we are playing the game
	if(GameState != PLAYING)
		return;

	
	


    /* clear the screen to the clear colour */
    /****************************************/

    glClear(GL_COLOR_BUFFER_BIT);

    /* draw everything */
    /*******************/
        
	Rocket->draw();
	Root->drawAsteroids();

	if(AsteroidDestroyed)
	{
		delete [] AsteroidText;
		//printf("current asteroid text: %s\n",AsteroidText);
		char *numAsts = intToChar(NumAsteroidsDestroyed);
		//printf("numAsts: %s\n",numAsts);
		AsteroidText = new char[strlen("Asteroids = ") + strlen(numAsts) + 1];
		strcpy(AsteroidText,"Asteroids = ");
		strcpy(&AsteroidText[strlen("Asteroids = ")],numAsts);
		//printf("new asteroid text: %s\n",AsteroidText);	
		
		AsteroidDestroyed = false;
	}

	if(ShipDestroyed)
	{

		delete [] ShipText;
		//printf("current ShipText : %s\n",ShipText);
		char *numShps = intToChar(NumShipsDestroyed);
		//printf("numAsts: %s\n",numShps);
		ShipText = new char[strlen("Rockets = ") + strlen(numShps) + 1];
		strcpy(ShipText,"Rockets = ");
		strcpy(&ShipText[strlen("Rockets = ")],numShps);
		//printf("new ShipText: %s\n",ShipText);	
		
		ShipDestroyed = false;
	}
	
	
	printText(LEFT_EDGE + 10,TOP_EDGE + 10,.07,AsteroidText,RIGHT);
	printText(LEFT_EDGE + 10,TOP_EDGE + 20,.07,ShipText,RIGHT);
	

    /* swap buffers */
    /****************/
    
    //prevTime = glutGet(GLUT_ELAPSED_TIME);
        
    glutSwapBuffers();

	
}

/*************************************************************
 *
 * readShipFile
 * 
 * code to read a ship file given by a name
 *************************************************************/
CVertexList* readShipFile(char *filename)
{
	if(!filename)
	{
		printf("No file name given!\n");
		filename = new char[strlen("ship.txt") + 1];
		strcpy(filename,"ship.txt");
		printf("default file %s\n",filename);	
	}
	
	FILE *shipFile;
	shipFile = fopen(filename,"r");
	if(!(shipFile))
	{
		printf("error reading file");
		return NULL;
	}
	CVertexList *vL = new CVertexList();
	
	CVertex *tempV = NULL;
	float tempx, tempy;
	
	printf("scanning file");
	while(fscanf(shipFile, "%f %f",	&tempx, &tempy) != EOF)
	{
		printf("%f %f\n",tempx,tempy);
		vL->addVertex((tempV = new CVertex(tempx,tempy)));
		
	}
	
	
	return vL;



}


/*************************************************************
 *
 * readAsteroidFile
 * 
 * code to read a ship file given by a name
 *************************************************************/
CVertexList* readAsteroidFile(char *filename)
{
	if(!filename)
	{
		printf("No file name given!\n");
		filename = new char[strlen("ast.txt") + 1];
		strcpy(filename,"ast.txt");
		printf("default file %s\n",filename);	
	}
	
	FILE *shipFile;
	shipFile = fopen(filename,"r");
	if(!(shipFile))
	{
		printf("error reading file");
		return NULL;
	}
	CVertexList *vL = new CVertexList();
	
	CVertex *tempV = NULL;
	float tempx, tempy;
	
	printf("scanning file");
	while(fscanf(shipFile, "%f %f",	&tempx, &tempy) != EOF)
	{
		printf("%f %f\n",tempx,tempy);
		vL->addVertex((tempV = new CVertex(tempx,tempy)));
		
	}
	
	
	return vL;



}

/*************************************************************
 *
 * dealWithParameters
 * 
 * here you could put some code to deal with the command-line
 * parameters for the program
 *************************************************************/
 
CWorld* dealWithParameters(int argc, char **argv)
{
	srand(time(NULL));
	TransformMatrixStack.top = getIdentityMatrix();
    	SET_ROCKET_TURN_RATE( 60 );		
    		enum arguement_state
		{
    			NO_ARG = 0,
    			SHIP_FILE,
    			ASTEROID_FILE,
    			SHIP_COLOR,
			ASTEROID_COLOR,
			MISSLE_COLOR,
			NUM_AST,
			SHIP_TURN_RATE,
			SPEED_ROCKET,
			SPEED_ASTEROID,
			SPEED_MISSLE
    		
    		
    		}argState;
    		
    		
    		argState = NO_ARG;
    		CVertexList *temp = NULL;
    		
    		
    		for(int i = 1; i < argc; i++)
    		{
    			switch(argState)
    			{
    			case NO_ARG:
    				if(!strcmp(argv[i],"-sf"))
    					argState = SHIP_FILE;
    				else if(!strcmp(argv[i],"-af"))
    					argState = ASTEROID_FILE;
				else if(!strcmp(argv[i],"-sc"))
					argState = SHIP_COLOR;
				else if(!strcmp(argv[i],"-ac"))
					argState = ASTEROID_COLOR;
				else if(!strcmp(argv[i],"-mc"))
					argState = MISSLE_COLOR;
				else if(!strcmp(argv[i],"-an"))
					argState = NUM_AST;
				else if(!strcmp(argv[i],"-tr"))
					argState = SHIP_TURN_RATE;
				else if(!strcmp(argv[i],"-ss"))
					argState = SPEED_ROCKET;
				else if(!strcmp(argv[i],"-as"))
					argState = SPEED_ASTEROID;
				else if(!strcmp(argv[i],"-ms"))
					argState = SPEED_MISSLE;

    				break;
    			default:
    				break;
    			
    			 			
    			}
    			
    			switch(argState)
				{
				case NO_ARG:
					printf("argument was not valid\n");
					break;
				case SHIP_FILE:
					printf("Setting ship file\n");
					if(Rocket)
						delete Rocket;
					
					Rocket = new CRocket((temp = readShipFile(argv[i + 1])));
					argState = NO_ARG;
					break;
				case ASTEROID_FILE:
					if(DefaultAsteroidVerts)
						delete DefaultAsteroidVerts;
						
					DefaultAsteroidVerts = readAsteroidFile(argv[i + 1]);
					argState = NO_ARG;
					break;
				case NUM_AST:
					NUM_ASTEROIDS = atoi(argv[i + 1]);
					argState = NO_ARG;
					break;
				case SHIP_COLOR:
					printf("getting ship color\n");
					if(argc < i + 3)
						break;
				
					for(int j = i+1; j< i+4; j++)
					{	
						printf("argv[%d] = %f\t",j,(float)atoi(argv[j]));
						RocketColor[j - i - 1] = (float)(atoi(argv[j]) % 256) / 255;
					}

					printf("\n");
					argState = NO_ARG;
					break;
				case ASTEROID_COLOR:
					printf("getting asteroid color\n");
					if(argc < i + 3)
						break;
				
					for(int j = i+1; j< i+4; j++)
					{	
						printf("argv[%d] = %f\t",j,(float)atoi(argv[j]));
						AsteroidColor[j - i - 1] = (float)(atoi(argv[j]) % 256) / 255;
					}

					printf("\n");
					argState = NO_ARG;
					break;
				case MISSLE_COLOR:
					printf("getting missle color\n");
					if(argc < i + 3)
						break;
				
					for(int j = i+1; j< i+4; j++)
					{	
						printf("argv[%d] = %f\t",j,(float)atoi(argv[j]));
						MissleColor[j - i - 1] = (float)(atoi(argv[j]) % 256) / 255;
					}

					printf("\n");
					argState = NO_ARG;
					break;

				case SHIP_TURN_RATE:
					if(argc < i + 1)
						break;

					SET_ROCKET_TURN_RATE((float)atoi(argv[i+1]));
					argState = NO_ARG;
					break;
				case SPEED_ROCKET:
					if(argc < i + 1)
						break;

					SET_ROCKET_MAX_SPEED((float)atoi(argv[i+1]));
					argState = NO_ARG;
					break;
				case SPEED_ASTEROID:
					if(argc < i + 1)
						break;

					SET_ASTEROID_SPEED((float)atoi(argv[i+1]));
					argState = NO_ARG;
					break;
				case SPEED_MISSLE:
					if(argc < i + 1)
						break;

					MISSLE_SPEED = UNITS_PER_SECOND((float)atoi(argv[i+1]));
					argState = NO_ARG;
					break;
				default:
					argState = NO_ARG;
					break;
				}	
    		
    		
    		
    		
    		}

    	//make new world with number of asteroids specified by NUM_ASTEROIDS with vertices defined by defaultAsteroidVerts
    	CWorld *result = new CWorld();
    	
    	if(!DefaultAsteroidVerts)
    		DefaultAsteroidVerts = readAsteroidFile(NULL);
    		
    	for(unsigned int i = 0; i < NUM_ASTEROIDS; i++)
    	{
		CAsteroid* temp = new CAsteroid(DefaultAsteroidVerts);
		printf("new asteroid at %d\n",temp);
		temp->setPosition((abs(rand()) % RIGHT_EDGE) * (abs(rand()) % 2 + -1),
					(abs(rand()) % BOTTOM_EDGE) * (abs(rand()) % 2 + -1));
		result->addAsteroid(temp);
		temp->setSpeed(ASTEROID_SPEED);
	}	


		printf("turn rate: %f\n",TURN_RATE);
					
	return result;				

}

/*************************************************************
 *
 * main
 * 
 *************************************************************/

int main(int argc, char *argv[])
{
	GameState = GAME_LOADING;
	
	SET_ROCKET_TURN_RATE( 60 );
	
	//Deal with command line parameters and get a world ready
	Root = dealWithParameters(argc,argv);
	
	CVertexList *temp; 
	
	
	
	if(!Rocket)
	{
		Rocket = new CRocket((temp = readShipFile(NULL)));
	}
	
	Rocket->setAssocWorld(Root);
	Rocket->setMissleSpeeds(MISSLE_SPEED);
	Root->setAssocRocket(Rocket);
	Rocket->setMaxSpeed(ROCKET_MAX_SPEED);
	
	//setup text to display on screen concerning number of asteroids or ship destroyed
	AsteroidText = new char[strlen("Asteroids = 0") + 1];
	strcpy(AsteroidText,"Asteroids = 0");
	ShipText = new char[strlen("Rockets = 0") + 1];
	strcpy(ShipText,"Rockets = 0");

	/* deal with any GLUT command Line options */
	/*******************************************/
       
	glutInit(&argc, argv);
    
	
	/* create an output window */
	/***************************/

	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);

	// set up the physical window size

    glutInitWindowSize(500, 500);

	// set the name of the window and try to create it

	glutCreateWindow("CS 488 - HW 2 - Arun Gangadhar Gudur Rao : arao@cs.uic.edu");

    if (glutGet(GLUT_WINDOW_COLORMAP_SIZE) != 0)
	{
	printf("FAIL: bad RGBA colormap size\n");
	exit(1);
	}

    /* set up for accepting keyboard commands */
    /******************************************/
    
    glutKeyboardFunc(keyboard);

    /* set up the logical graphics space */
    /*************************************/
   
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(LEFT_EDGE, RIGHT_EDGE, BOTTOM_EDGE, TOP_EDGE);
    glClearColor(0.0, 0.0, 0.0, 1.0);

    /* assign the display function */
    /*******************************/
    
    glutDisplayFunc(display);

    /* assign the idle function */
    /****************************/

    glutIdleFunc(display);

    /* set up a timer to regularly update the scene */
    /************************************************/

    glutTimerFunc(UPDATE_TIME, update, 1);
    glutTimerFunc(MISSLE_UPDATE_TIME, missleUpdate, 1);	
    /* set everything going */
    /************************/
   // glLoadIdentity();
		
	GameState = PLAYING;

    glutMainLoop();
    
    delete TransformMatrixStack.top;

    
  return 0;  
}

