#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include <string.h>

//most systems include GL/glut.h others just glut.h
#include "glut.h"


////////////datatype declerations

//struct STransform;
//struct STransformStack_t;

struct STransform
{
	float matrix[3][3];
	
	STransform *next;

};

struct STransformStack_t
{
	STransform *top;
}TransformMatrixStack;

enum GameState_t
{
GAME_LOADING = 0,
PLAYING,
END_SCREEN
} GameState;

class CVertex;
class CVertexList;
class CSpaceObject;
class CRocket;
class CAsteroid;
class CMissle;
class CWorld;


// Set up the logical borders 
/////////////////////////////
 
#define LEFT_EDGE -200
#define RIGHT_EDGE 200
#define TOP_EDGE -200
#define BOTTOM_EDGE 200
#define UP -1
#define LEFT -2
#define DOWN 1
#define RIGHT 2
#define RED 0
#define GREEN 1
#define BLUE 2
///////////////////////////////
//global variables, constants, enumeration types and macros
#define ATAN( n ) atanf( n )
#define COS( n ) cosf( n )
#define SIN( n ) sinf( n )
#define ACOS( n ) acosf( n )
#define TAN( n ) tanf( n )


#define X 0
#define Y 1
#define MISSLE_AMMO 3
#define ASTEROID_ROT_CCW 1 
#define ASTEROID_ROT_CW 0
#define ASTEROID_THETA atanf(5 * acos(-1) / 180)

#define FONT_SCALE_1 .5
#define FONT_SCALE_2 .8
unsigned int UNIT = 1;
unsigned int UPDATE_TIME = 10;
unsigned int MISSLE_UPDATE_TIME = 10;
#define SET_UNIT_LENGTH( l ) ((void)(UNIT = ((float) l)))
#define UNITS_PER_SECOND( unitsPerSecond ) (((float) (unitsPerSecond) * UNIT)/((float)(1000/UPDATE_TIME)))

//the number of radians (+/-) per second
float TURN_RATE;
bool CCROTATE = false;
#define SET_ROCKET_TURN_RATE( th ) ( TURN_RATE = (((float)((th) /* (UNIT)*/) * ACOS(-1) / ((float)180))) * ((float)UPDATE_TIME * 3 / (float) 1000))
float ROCKET_MAX_SPEED = UNITS_PER_SECOND(200);
#define SET_ROCKET_MAX_SPEED( sp ) (ROCKET_MAX_SPEED = UNITS_PER_SECOND( sp ))
unsigned int INIT_NUMBER_ASTEROIDS = 3;
#define SET_INIT_NUMBER_ASTEROIDS( n ) ( INIT_NUMBER_ASTEROID = ((unsigned int)(n)))
unsigned int MISSLE_SPEED = UNITS_PER_SECOND(250);
unsigned int ASTEROID_SPEED = UNITS_PER_SECOND(100);
#define SET_ASTEROID_SPEED( sp ) ( ASTEROID_SPEED = UNITS_PER_SECOND( sp ))
unsigned int ASTEROID_UPDATE_TIME = 20;
unsigned int NUM_ASTEROIDS = 3;
CVertexList *DefaultAsteroidVerts = NULL;
float RocketColor[3]={0,.5,.75};
float AsteroidColor[3]={1,0,0};
float MissleColor[3]={1,1,0};
char BOOM_TEXT[] = "BOOM";
unsigned int NumShipsDestroyed = 0;
unsigned int NumAsteroidsDestroyed = 0;
unsigned int NumAsteroidDestroyedByMissle = 0;
bool AsteroidDestroyed = false;
bool ShipDestroyed = false;
char *ShipText = NULL;
char *AsteroidText = NULL;
unsigned int NumberShots = 0;
int ShotPercentage = 0;
unsigned int TotalScore = 0;
clock_t RocketRotateStop;
int prevTime, currentTime;

////////////////////////////// Global variable!!!!  Root of world///////////////////////



			//STransfromStack_t TransformMatrixStack;
			CWorld* Root = NULL;
			CRocket* Rocket = NULL;



//////////////////////////////////////////////////////////////////////////////////////////

///////////function prototypes
void update(int value);
void missleUpdate(int value);
void display(void);
void keyboard(unsigned char c, int x, int y);
float getAngleRad(float*& pos, float *& org);	//pos is the position and org is the point of reference
void loadIdentity(void);
STransform * getIdentityMatrix(void);
void scaleMatrix(float x,float y);
void translateMatrix(float x, float y);
void rotateMatrix(float theta);
void pushMatrix(void);
void popMatrix(void);
float* transformPoint(STransform& a, const float*& b);
void printText(float x, float y, float scale, char* text,int dir);
char* intToChar(int num);

//gameState
enum GameState
{
LOADING_GAME = 0,
LOADING_SHIP,
LOADING_ASTEROIDS,
RUNNING,
PAUSED,
MAIN_MENU,
ROCKET_SPEED_OPTION,
ROCKET_ROTATE_SPEED_OPTION,
MISSLE_SPEED_OPTION,
ASTEROID_SPEED_OPTION,
NUMBER_ASTEROIDS_OPTION,
SHIP_COLOR_OPTION,
ASTEROID_COLOR_OPTION,
TEXT_COLOR_OPTION,
SHIP_FILE_OPTION,
ASTEROID_FILE_OPTION,
DRAW_SHIP_MENU


};

enum AsteroidDamageLevel
{
LEVEL0 = 0,
LEVEL1,
DESTROYED


};




////////////////////////////////////////////////////////////////////////////////////////////////////
class CVertex //class to represent the vertices of objects in game
{
private:
	float *pos;	//current x y positions of the vertex
	float magFromZeroSquared;	//the distance the vertex is supposed to be from a particular origin, squared
	CVertex *next;
	
	
	/////////////////////////////////////////////////////////
	inline void setNext(CVertex *v){ next = v; }
	inline CVertex * getNext(void) { return next; }

	
public:
	friend class CVertexList;
	
	
	CVertex(float posX, float posY);
	
	CVertex(const CVertex& v);
	
	~CVertex();

	inline void setPosition(float posX,float posY) { pos[X] = posX; pos[Y] = posY;}
	inline void setPosition(float* temp){ pos[X] = temp[X]; pos[Y] = temp[Y];}
	
	//get the position of the vertex in an array
	inline const float*& getPosition(void){return pos;}
	
	inline float getX(void) { return pos[X]; }
	inline float getY(void) { return pos[Y]; }
	inline float getMagFromZero(void) { return sqrt(magFromZeroSquared); }
	
	
	void scale(float scaleV);
};

////////////////////////////////////////////////////////////////////////////////////////////////////
class CVertexList
{
private:
	CVertex *head, *tail;
	int length;


	
public:
	CVertexList(void);
	
	CVertexList(const CVertexList& vList);
	
	
	
	~CVertexList();
	
	//add another vertex to the end of the list
	void addVertex(CVertex *v);
	//get a vertex at a specific index in the list
	CVertex * getVertex(int i);
	
	//get a vertex after a given vertex
	inline CVertex * getNextVertex(CVertex *v) { return v->getNext(); }
	
	inline int getLength(void){ return length;}
	
	void scaleList(float scaleV);
	
	
	
};

////////////////////////////////////////////////////////////////////////////////////////////////////
class CSpaceObject	//abstract class to be used for inheritance purposes
{
protected:
	CVertexList *vertexList;
	float avgRadius;
	float xVelocity, yVelocity;
	CVertex *position;
	CVertex *direction;
	CWorld* assocWorld;
	bool canTakeDamage;
	float red,green,blue;

	void copyVertexList(CVertexList *vL);
	
public:
	
	CSpaceObject();
	
	virtual inline void setAssocWorld(CWorld* worldPtr){ assocWorld = worldPtr; }
	virtual inline CVertex* getCenter(void){return position;}
	virtual inline float getAvgRadius(void){return avgRadius;}
	virtual void move() {};
	virtual inline void setColor(float r, float g, float b){ red = r; green = g; blue = b;}
	virtual inline void setPosition(float x, float y){ position->setPosition(x,y);}
	virtual void setSpeed(float sp);
};



////////////////////////////////////////////////////////////////////////////////////////////////////
class CAsteroid : public CSpaceObject //"asteroid" class to represent asteroid objects
{
private:
	char rotateDirection;
	bool collidedWithMissle;
	
	AsteroidDamageLevel damageLevel;
	float theta;
	CAsteroid *next;
public:
	CAsteroid();
	
	CAsteroid(CVertexList *vList);
	inline CAsteroid* getNext(){ return next; }
	inline void setNext(CAsteroid* n){ next = n; }
	inline void setRotation(int rDir){ rotateDirection = abs(rDir) % 2;}
	inline void setDamageLevel(AsteroidDamageLevel lvl){ damageLevel = lvl; }
	inline AsteroidDamageLevel getDamageLevel( void ) { return damageLevel; }
	
	virtual void move();
	
	
	void takeDamage( void );
	
	
	inline bool isDamaged(void) { return collidedWithMissle; }
	 
	CAsteroid* spawnChildren(void);
	void draw();

};


////////////////////////////////////////////////////////////////////////////////////////////////////
class CWorld	//"world" class to hold objects to be rendered and interacted with
{

private:
	CAsteroid *asteroidList;
	CRocket *assocRocket;
	

public: 
	CWorld();
	
	bool addAsteroid(CAsteroid* ast);
	
	inline CAsteroid* getAsteroids(void)
	{ return asteroidList; }
	
	inline void setAssocRocket(CRocket *rkt) { assocRocket = rkt; }

	void updateAsteroidList( void );
	
	void drawAsteroids(void);

	void moveAsteroids(void);
	
	
	
};

////////////////////////////////////////////////////////////////////////////////////////////////////
class CMissle : public CSpaceObject
{
private:
float pos[2];
float boomTextPos[2];
int textDir;
float speed;
bool inMissleBay;
float expRed, expGreen, expBlue;
time_t animEndTime;

enum MissleAnimState_t
{
	NO_EXPLODE = 0,
	EXPLODE_ANIM,
	EXPLODE_DONE,
} missleAnimState;

public:

	CMissle( );
	
	inline void setSpeed(float sp){ speed = sp; }
	
	inline void setColor(float r, float g, float b)
	{	red = r; green = g; blue = b; }
		 
	bool fire( float x, float y, float xDir, float yDir );
	
	inline bool inBay( void ) { return inMissleBay; }
	
	void draw(void);
	
	//move the missle
	void move(void);

	void setExplosionColor(void);
};

////////////////////////////////////////////////////////////////////////////////////////////////////
class CRocket: public CSpaceObject //"rocket" class to represent the rocket in the game
{
private:
	
	
	CMissle* missleBay[MISSLE_AMMO];
	time_t	endInvincibleTime;
	bool invincible;
	float maxSpeed;
public:
	CRocket(CVertexList*& vL);
	
	inline void setMaxSpeed(float sp){ maxSpeed =  sp; }
	
	//set the associated world and set the assocworld of the missles
	virtual void setAssocWorld(CWorld *wPtr);
	void draw(void);
	
	
	
	virtual void move();
	
	//accelerate in the direction the ship is facing
	void thrustInDirection();

	//rotate the rocket about its own origin
	void rotateRocket(float theta);
	
	//shoot an available missle
	void shootMissle(void);
	
	void updateMissles(void);

	void setMissleSpeeds(float sp);

	void temporaryInvincible(void);

};
